/*
This is a part of the LiteStep Shell Source code.

Copyright (C) 1997-2001 The LiteStep Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/****************************************************************************
01-17-2000 - Charles Oliver Nutter (Headius)
  - Added TaskbarWidth (default of screen width), TaskbarPositioning (default
    of false, specifies that TaskbarX and TaskbarY should be used), TaskbarX
	(default of 0), TaskbarY (default of 0)
12-06-2000 - Kevin Schaffer (Maduin)
  - Added TaskbarFont (et al) and TaskbarActiveFont (et al)
11/28/00 - Bobby G. Vinyard (Message)
  - Added multimonitor support
11/24/00 - Richard Lin (Chaku)
  - Fixed the start button text position problem, and the non-reponsive mouse
    clicking when start button is absent
11/22/00 - Richard Lin (Chaku)
  - Added task flashing.
11/15/00 - Kevin Schaffer (Maduin)
  - First version
****************************************************************************/

#include "common.h"
#include <commctrl.h>
#include <algorithm>
#include "../lsapi/lsapi.h"
#include "../lsapi/macros.h"
#include "taskbar.h"

#define WC_TASKBAR		"Taskbar"
#define EDGE_BOTTOM		0
#define EDGE_TOP		1
#define TIMER_AUTOHIDE	1
#define TIMER_UPDATE    2
#define BUFFER_SIZE		1024

#define SIZE_NO			0
#define SIZE_LEFT		1
#define SIZE_RIGHT		2

// bang commands
void hideBangCommand(HWND hCaller, const char *args);
void toggleBangCommand(HWND hCaller, const char *args);
void showBangCommand(HWND hCaller, const char *args);
void sizeBangCommand(HWND hCaller, const char *args);
void moveBangCommand(HWND hCaller, const char *args);
void linesBangCommand(HWND hCaller, const char *args); 

void toggleHideBangCommand(HWND hCaller, const char *args); 

// undocumented SwitchToThisWindow API (maybe this should be in LSAPI?)
void (WINAPI *SwitchToThisWindow)(HWND, int);

const char aboutStr[] = "Taskbar.dll: 0.207b2 (jesus_mjjg)";

Taskbar *taskbar;

//ignore
vector<string> IgnoreList;
void ReadTaskbarIgnore(void);
BOOL IsOnTaskbarIgnoreList(HWND hWnd);
//end ignore
//hidden
BOOL removeHiddenHwnd;
vector<string> HiddenList;
void ReadTaskbarHidden(void);
BOOL IsOnTaskbarHiddenList(HWND hWnd);
//end hidden

//HWND systrayHwnd;

int initModuleEx(HWND hParent, HINSTANCE hInstance, LPCSTR pszPath)
{
	InitCommonControls();

	SwitchToThisWindow = (void (WINAPI *)(HWND, int)) GetProcAddress(
	                         GetModuleHandle("USER32.DLL"),
	                         "SwitchToThisWindow");

    ReadTaskbarIgnore();//ignore
	ReadTaskbarHidden();//ignore

	taskbar = new Taskbar;
	taskbar->OnLoad(hInstance);

	AddBangCommand("!TaskbarHide", hideBangCommand);
	AddBangCommand("!TaskbarToggle", toggleBangCommand);
	AddBangCommand("!TaskbarShow", showBangCommand);
	AddBangCommand("!TaskbarSize", sizeBangCommand);
	AddBangCommand("!TaskbarMove", moveBangCommand);
	AddBangCommand("!TaskbarLines", linesBangCommand);

	AddBangCommand("!TaskbarToggleHidden", toggleHideBangCommand);

	return 0;
}

void quitModule(HINSTANCE hInstance)
{
	RemoveBangCommand("!TaskbarHide");
	RemoveBangCommand("!TaskbarToggle");
	RemoveBangCommand("!TaskbarShow");

    RemoveBangCommand("!TaskbarSize");
    RemoveBangCommand("!TaskbarMove");
    RemoveBangCommand("!TaskbarLines");

	RemoveBangCommand("!TaskbarTest");

	taskbar->OnUnload();
	delete taskbar;
}

extern "C" int WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID pvReserved)
{
	if (dwReason == DLL_PROCESS_ATTACH)
		DisableThreadLibraryCalls(hInstance);

	return 1;
}

//
//ignore
//

BOOL IsOnTaskbarIgnoreList(HWND hWnd){
	char tmpTitle[BUFFER_SIZE], tmpClass[BUFFER_SIZE];

	GetWindowText(hWnd, tmpTitle, 1024);
	GetClassName(hWnd, tmpClass, 1024);
    vector<string>::iterator it;

	if (!lstrcmpi(tmpClass, "WindowsScreenSaverClass") || !lstrcmpi(tmpClass, "tooltips_class32"))
		return TRUE;
	for (it = IgnoreList.begin();it != IgnoreList.end();++it){
		if ((*it == tmpTitle) || (*it == tmpClass)) //it has to be exactly the same, no substrings atm
			return TRUE;
	}
	return FALSE;
}

BOOL IsOnTaskbarHiddenList(HWND hWnd){
	char tmpTitle[BUFFER_SIZE], tmpClass[BUFFER_SIZE];

	GetWindowText(hWnd, tmpTitle, 1024);
	GetClassName(hWnd, tmpClass, 1024);
    vector<string>::iterator it;

	if (!lstrcmpi(tmpClass, "WindowsScreenSaverClass") || !lstrcmpi(tmpClass, "tooltips_class32"))
		return TRUE;
	for (it = HiddenList.begin();it != HiddenList.end();++it){
		if ((*it == tmpTitle) || (*it == tmpClass)) //it has to be exactly the same, no substrings atm
			return TRUE;
	}
	return FALSE;
}

BOOL IsSystrayHwnd(HWND hWnd){
	char tmpTitle[BUFFER_SIZE], tmpClass[BUFFER_SIZE];

	GetWindowText(hWnd, tmpTitle, 1024);
	GetClassName(hWnd, tmpClass, 1024);

	if ((strstr(tmpTitle, "systray")) && (strstr(tmpClass, "systray")))
		return TRUE;
	return FALSE;
}

void ReadTaskbarIgnore(void){
    //code from www.mindjunction.com
	FILE *f;
	f = LCOpen (NULL);
	if (f){
		char buffer[BUFFER_SIZE];
 		char token1[1204], token2[BUFFER_SIZE], extra_text[BUFFER_SIZE];
 		char* tokens[2];
	 
 		tokens[0] = token1;
 		tokens[1] = token2;
 
	 	buffer[0] = 0;
 		while (LCReadNextConfig (f, "*TaskbarIgnore", buffer, sizeof (buffer)))
 		{
  			int count;
 
  			token1[0] = token2[0] = extra_text[0];
  
  			count = LCTokenize (buffer, tokens, 2, extra_text);
  
  			// token 1 is "*TaskbarIgnore", token 2 is the
  			// application class or title, and the extra_text are something to forget.
  			if (count >= 2)
				IgnoreList.push_back(tokens[1]);
 		}
 		LCClose(f);

		for (int i = 0; i < 2; i++)
			delete [] tokens[i];
		}
		return;
}

void ReadTaskbarHidden(void){
    //code from www.mindjunction.com
	FILE *f;
	f = LCOpen (NULL);
	if (f){
		char buffer[BUFFER_SIZE];
 		char token1[1204], token2[BUFFER_SIZE], extra_text[BUFFER_SIZE];
 		char* tokens[2];
	 
 		tokens[0] = token1;
 		tokens[1] = token2;
 
	 	buffer[0] = 0;
 		while (LCReadNextConfig (f, "*TaskbarHidden", buffer, sizeof (buffer)))
 		{
  			int count;
 
  			token1[0] = token2[0] = extra_text[0];
  
  			count = LCTokenize (buffer, tokens, 2, extra_text);
  
  			// token 1 is "*TaskbarHidden", token 2 is the
  			// application class or title, and the extra_text are something to forget.
  			if (count >= 2)
				HiddenList.push_back(tokens[1]);
 		}
 		LCClose(f);

		for (int i = 0; i < 2; i++)
			delete [] tokens[i];
		}
		return;
}

// bang commands
//

void toggleHideBangCommand(HWND hCaller, const char *args)
{
	if(removeHiddenHwnd == TRUE)
		removeHiddenHwnd = FALSE;
	else
		removeHiddenHwnd = TRUE;
	taskbar->Layout();
}

void hideBangCommand(HWND hCaller, const char *args)
{
	taskbar->hide();
}

void toggleBangCommand(HWND hCaller, const char *args)
{
	if (taskbar->isVisible())
		taskbar->hide();
	else
		taskbar->show();
}

void showBangCommand(HWND hCaller, const char *args)
{
	taskbar->show();
}


void sizeBangCommand(HWND hCaller, const char *args){
    //code from www.mindjunction.com
	int cx = 0, cy = 0;
    char token1[BUFFER_SIZE], token2[BUFFER_SIZE], extra_text[BUFFER_SIZE];
    char* tokens[11];
    int count;
    tokens[0] = token1;
    tokens[1] = token2; 

    count = LCTokenize (args, tokens, 2, extra_text);

    cx = atoi(token1);
    cy = atoi(token2);

    taskbar->OnReSize( cx, cy );
}

void moveBangCommand(HWND hCaller, const char *args){
    //code from www.mindjunction.com
	int cx = 0, cy = 0;
    char token1[BUFFER_SIZE], token2[BUFFER_SIZE], extra_text[BUFFER_SIZE];
    char* tokens[11];
    int count;
    tokens[0] = token1;
    tokens[1] = token2; 

    count = LCTokenize (args, tokens, 2, extra_text);

    cx = atoi(token1);
    cy = atoi(token2);

    taskbar->OnMove( cx, cy );
}

void linesBangCommand(HWND hCaller, const char *args){
    //code from www.mindjunction.com
	int lines = 0;
    char token1[BUFFER_SIZE], extra_text[BUFFER_SIZE];
    char* tokens[11];
    int count;
    tokens[0] = token1;

    count = LCTokenize (args, tokens, 1, extra_text);

    lines = atoi(token1);

	taskbar->taskbarLines =  lines;
	if (taskbar->taskbarLines <= 0)
	  taskbar->taskbarLines = 1;

    taskbar->Layout();

}

// utility functions
//

HFONT GetRCFont(const char *name, const LOGFONT *lfDefault)
{
	LOGFONT lf;
	char configName[32];

	//memcpy(&lf, lfDefault, sizeof(LOGFONT));

	if (GetRCBool(name, TRUE))
	{
		memset(&lf, 0, sizeof(LOGFONT));

		wsprintf(configName, "%s", name);
		if (GetRCBool(configName, TRUE))
			GetRCString(configName, lf.lfFaceName, lfDefault->lfFaceName, LF_FACESIZE);

		wsprintf(configName, "%sHeight", name);
		if (GetRCBool(configName, TRUE))
			lf.lfHeight = GetRCInt(configName, lfDefault->lfHeight);

		wsprintf(configName, "%sBold", name);
		if (GetRCBool(configName, TRUE))
			lf.lfWeight = GetRCBool(configName, TRUE) ? FW_BOLD : FW_NORMAL;

		wsprintf(configName, "%sItalic", name);
		if (GetRCBool(configName, TRUE))
			lf.lfItalic = GetRCBool(configName, TRUE);
	}
	else
	{
		memcpy(&lf, lfDefault, sizeof(LOGFONT));
	}

	return CreateFontIndirect(&lf);
}

BOOL IsAppWindow( HWND hWnd )
{
	if ( !IsWindowVisible( hWnd ) )
		return FALSE;

	if ( GetWindow( hWnd, GW_OWNER ) )
		return FALSE;

	LONG styleEx = GetWindowLong( hWnd, GWL_EXSTYLE );

	if ( styleEx & WS_EX_APPWINDOW )
		return TRUE;

	if ( styleEx & WS_EX_TOOLWINDOW )
		return FALSE;

	if ( GetWindowLong( hWnd, GWL_USERDATA ) == 0x49474541 )
		return FALSE;

	return TRUE;
}

// from jugg's tasks.dll, based on code from Fahim and re5ource
HICON GetIconFromWindow( HWND hWnd, BOOL bBigIcon )
{
	HICON hIcon = NULL;

	if (hWnd)
	{
		if ( bBigIcon )
		{
			SendMessageTimeout( hWnd, WM_GETICON, ICON_BIG, 0, SMTO_ABORTIFHUNG, 1000, (LPDWORD) &hIcon );
			if ( !hIcon )
				hIcon = (HICON) GetClassLong( hWnd, GCL_HICON );
			if ( !hIcon )
				SendMessageTimeout( hWnd, WM_QUERYDRAGICON, 0, 0, SMTO_ABORTIFHUNG, 1000, (LPDWORD) &hIcon );
			if ( !hIcon )
				SendMessageTimeout( hWnd, WM_GETICON, ICON_SMALL, 0, SMTO_ABORTIFHUNG, 1000, (LPDWORD) &hIcon );
			//if( !hIcon ) hIcon = (HICON) GetClassLong( hWnd, GCL_HICONSM );
			if ( !hIcon )
				hIcon = (HICON) GetClassLong( hWnd, GCL_HICON );
		}
		else
		{
			SendMessageTimeout( hWnd, WM_GETICON, ICON_SMALL, 0, SMTO_ABORTIFHUNG, 1000, (LPDWORD) &hIcon );
			if ( !hIcon )
				hIcon = (HICON) GetClassLong( hWnd, GCL_HICONSM );
			if ( !hIcon )
				SendMessageTimeout( hWnd, WM_QUERYDRAGICON, 0, 0, SMTO_ABORTIFHUNG, 1000, (LPDWORD) &hIcon );
			if ( !hIcon )
				SendMessageTimeout( hWnd, WM_GETICON, ICON_BIG, 0, SMTO_ABORTIFHUNG, 1000, (LPDWORD) &hIcon );
			//if( !hIcon ) hIcon = (HICON) GetClassLong( hWnd, GCL_HICON );
			if ( !hIcon )
				hIcon = (HICON) GetClassLong( hWnd, GCL_HICONSM );
		}
	}

	return hIcon;
}

BOOL WINAPIV PrintLog( LPCSTR pszFormat, ... )
{
	char szText[BUFFER_SIZE];
	DWORD dwCount;
	va_list argList;
	HANDLE hFile;

	va_start( argList, pszFormat );
	wvsprintf( szText, pszFormat, argList );
	lstrcat( szText, "\r\n" );

	hFile = CreateFile( "C:\\LSDISTRO\\TASKBAR.LOG",
	                    GENERIC_WRITE,
	                    FILE_SHARE_READ | FILE_SHARE_WRITE,
	                    NULL,
	                    OPEN_ALWAYS,
	                    FILE_ATTRIBUTE_NORMAL,
	                    NULL );

	SetFilePointer( hFile, 0, NULL, FILE_END );
	WriteFile( hFile, szText, lstrlen( szText ), &dwCount, 0 );

	CloseHandle( hFile );
	return TRUE;
}

//old taskbar.cpp

//
// Taskbar
//

Taskbar::Taskbar()
{
	hWnd = NULL;
	hInstance = NULL;
	hToolTips = NULL;

	edge = EDGE_BOTTOM;
	taskbarTrayResize = SIZE_NO;
	height = 0;
	width = 0;

	neededHeight = 0;

	screenHeight = 0;
	screenWidth = 0;

	active = FALSE;
	autoHidden = FALSE;
	mouseOver = FALSE;
	capture = NULL;

	skin = NULL;
	buttonSkin = NULL;
	activeButtonSkin = NULL;
	flashSkin = NULL;
	minimizedSkin = NULL;

	activeFont = 0;
	minimizedFont = 0;
	flashFont = 0;
	font = 0;

    maxTaskWidth = SCREEN_WIDTH;

	borderLeft = 4;
	borderTop = 4;
	borderRight = 4;
	borderBottom = 4;

	buttonBorderLeft = 4;
	buttonBorderTop = 4;
	buttonBorderRight = 4;
	buttonBorderBottom = 4;

	spacing = 3;
	taskbarHeight = 0;

	taskbarLines = 1;

    iconSize = 16;
    iconSpacing = 3;


}

Taskbar::~Taskbar()
{
	// ...
}

UINT messages[] = {
                      LM_LSSELECT,
                      LM_GETMINRECT,
                      LM_REDRAW,
                      LM_WINDOWACTIVATED,
                      LM_WINDOWCREATED,
                      LM_WINDOWDESTROYED,
					  LM_GETREVID,
                      0
                  };

void Taskbar::OnLoad( HINSTANCE hInstanceP )
{
	hInstance = hInstanceP;

	// get config variables from step.rc
	ReadConfig();

	// register taskbar window class
	WNDCLASSEX wc;
	memset( &wc, 0, sizeof(WNDCLASSEX) );

	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_GLOBALCLASS;
	wc.lpfnWndProc = Taskbar::WndProc;
	wc.cbWndExtra = sizeof(Taskbar *);
	wc.hInstance = hInstance;
	wc.hCursor = LoadCursor( NULL, IDC_ARROW );
	wc.lpszClassName = WC_TASKBAR;
	if ( !RegisterClassEx( &wc ) )
	{
		RESOURCE_MSGBOX( hInstance, IDS_TASKBAR_ERROR1,
		                 "Could not register taskbar window class.", "Taskbar")
		return ;
	}

	// create the taskbar window
	hWnd = CreateWindowEx( WS_EX_TOOLWINDOW,
	                       WC_TASKBAR,
						   WC_TASKBAR,
	                       WS_POPUP,
	                       0, 0, 0, 0,
	                       GetLitestepWnd(),
	                       NULL,
	                       hInstance,
	                       (LPVOID) this );

	if ( !hWnd )
	{
		RESOURCE_MSGBOX( hInstance, IDS_TASKBAR_ERROR2,
		                 "Could not create taskbar window.", "Taskbar")

		return ;
	}

	// create tooltips control
	hToolTips = CreateWindowEx( WS_EX_TOOLWINDOW | WS_EX_TOPMOST,
	                            TOOLTIPS_CLASS,
	                            NULL,
	                            TTS_ALWAYSTIP | WS_POPUP,
	                            0, 0, 0, 0,
	                            hWnd,
	                            NULL,
	                            hInstance,
	                            NULL );

	// adjust the Z-order, size, position
	SetWindowPos( hWnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE );
	
	// hide us from the VWM
	SetWindowLong( hWnd, GWL_USERDATA, magicDWord );

	// create font and compute needed height
	if ( autoHide )
		autoHidden = TRUE;

	OnSettingChange();

	// show the window
	visible = true;
	ShowWindow(hWnd, SW_SHOWNOACTIVATE);

	// adjust desktop area
	if (!autoHide && !keepDesktopArea)
	{
		RECT r;

		SystemParametersInfo(SPI_GETWORKAREA, 0, &r, 0);
		prevDeskArea = r;

		if (edge == EDGE_BOTTOM)
			r.bottom = r.bottom - neededHeight - 2;
		else
			r.top = r.top + neededHeight - 2;

		if (visible)
			SystemParametersInfo(SPI_SETWORKAREA, 0, &r, SPIF_SENDCHANGE);

		adjustedDeskArea = r;
	}

	// enumerate current tasks, setup buttons and start update timer
	EnumWindows(Taskbar::EnumWindowsProc, (LPARAM) this);
	Layout();
	//SetTimer(hWnd, TIMER_UPDATE, 2000, NULL);
	SetTimer(hWnd, TIMER_UPDATE, 500, NULL);

	// register for LS messages
	SendMessage( GetLitestepWnd(), LM_REGISTERMESSAGE, (WPARAM) hWnd, (LPARAM) ::messages );
}

void Taskbar::OnUnload()
{
	// shutdown timer
	KillTimer(hWnd, TIMER_UPDATE);

	// unregister messages
	SendMessage( GetLitestepWnd(), LM_UNREGISTERMESSAGE,
	             (WPARAM) hWnd, (LPARAM) ::messages );

	// reset desktop area
	if (!autoHide && !keepDesktopArea)
		SystemParametersInfo(SPI_SETWORKAREA, 0, &prevDeskArea, SPIF_SENDCHANGE);

	// destroy tooltips control
	if ( hToolTips )
		DestroyWindow( hToolTips );

	// destroy window and unregister class
	if ( hWnd )
		DestroyWindow( hWnd );

	UnregisterClass( WC_TASKBAR, hInstance );

	// release skins
	delete skin;
	delete buttonSkin;
	delete activeButtonSkin;
	delete flashSkin;
    delete minimizedSkin;
	//	delete activeFlashSkin;

	// release fonts
	DeleteObject(activeFont);
	DeleteObject(font);
	DeleteObject(minimizedFont);
	DeleteObject(flashFont);

	// release buttons
	for (TaskbarButtonIterator it = buttons.begin(); it != buttons.end(); ++it)
	{
		delete (*it);
		*it = NULL;
	}

	buttons.clear();
}

void Taskbar::setAlwaysOnTop(boolean alwaysOnTop)
{
	this->alwaysOnTop = alwaysOnTop;

	if (hWnd)
	{
		SetWindowPos(hWnd,
		             alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
		             0, 0, 0, 0,
		             SWP_NOMOVE | SWP_NOSIZE);
	}
}

void Taskbar::hide()
{
	if (visible)
	{
		visible = false;
		ShowWindow(hWnd, SW_HIDE);

		if (!autoHide && !keepDesktopArea)
		{
			SystemParametersInfo(SPI_GETWORKAREA, 0, &adjustedDeskArea, 0);
			SystemParametersInfo(SPI_SETWORKAREA, 0, &prevDeskArea, SPIF_SENDCHANGE);
		}
	}
}

void Taskbar::show()
{
	if (!visible)
	{
		visible = true;
		ShowWindow(hWnd, SW_SHOWNOACTIVATE);

		if (!autoHide && !keepDesktopArea)
		{
			SystemParametersInfo(SPI_GETWORKAREA, 0, &prevDeskArea, 0);
			SystemParametersInfo(SPI_SETWORKAREA, 0, &adjustedDeskArea, SPIF_SENDCHANGE);
		}
	}
}

void Taskbar::Invalidate( int x, int y, int cx, int cy )
{
	RECT r;

	r.left = x;
	r.top = y;
	r.right = x + cx;
	r.bottom = y + cy;

	InvalidateRect( hWnd, &r, FALSE );
}

TaskbarButton *Taskbar::GetCapture()
{
	return capture;
}

void Taskbar::SetCapture( TaskbarButton *button )
{
	::SetCapture( hWnd );
	capture = button;
}

void Taskbar::ReleaseCapture()
{
	::ReleaseCapture();
	capture = NULL;
}

TaskbarButton *Taskbar::ButtonAtPoint( int x, int y )
{
	for (TaskbarButtonIterator it = buttons.begin(); it != buttons.end(); ++it)
	{
		if ( (*it)->HasPoint(x, y) )
		{
			return (*it);
		}
	}

	return NULL;
}

void Taskbar::ReadConfig()
{
	char buffer[MAX_PATH];
	char bufferLeft[MAX_PATH];
	char bufferRight[MAX_PATH];

	keepDesktopArea = GetRCBool("TaskbarKeepDesktopArea", TRUE) ? true : false;

	// autohide settings
	autoHide = GetRCBool( "TaskbarAutoHide", TRUE );
	autoHideDelay = GetRCInt( "TaskbarAutoHideDelay", 500 );

	// flash
	isTaskFlash = GetRCBool( "TaskButtonFlash", TRUE) ? true : false;

	// edge
	if ( GetRCString( "TaskbarEdge", buffer, "", 16 ) )
	{
		if ( !stricmp( buffer, "TOP" ) )
			edge = EDGE_TOP;
	}

	if ( GetRCString( "TaskbarTrayResize", buffer, "", 16 ) ) //this is driving me nuts
	{
		if ( !stricmp( buffer, "LEFT" ) )
			taskbarTrayResize = SIZE_LEFT;
		else if ( !stricmp( buffer, "RIGHT" ) )
			taskbarTrayResize = SIZE_RIGHT;
  	    else
		    taskbarTrayResize = SIZE_NO;
	}
	else
		taskbarTrayResize = SIZE_NO;

	// appearance
	if( msTaskbar = GetRCBool( "TaskbarMSColors", TRUE ) );
	else{
	  foreColor1 = GetRCColor( "TaskbarFore", RGB( 192, 192, 192 ) );
 	  foreColor2 = GetRCColor( "TaskbarFore2", RGB( 64, 64, 64 ) );
	  backColor = GetRCColor( "TaskbarBack", RGB( 128, 128, 128 ) );
	  activeColor = GetRCColor( "TaskbarActiveColor", RGB( 128, 128, 128 ) );
	  minColor =  GetRCColor( "TaskbarMinColor", RGB( 128, 128, 128 ) );
	  flashColor = GetRCColor( "TaskbarFlash", RGB( 224, 224, 224 ) );

	  noSkinShift = GetRCBool( "TaskbarNoSkinShift", TRUE );
	  noTextShift = GetRCBool( "TaskbarNoTextShift", TRUE );

	  // taskbar skin
	  if ( GetRCString( "TaskbarSkin", buffer, "", MAX_PATH ) )
	  {
		  GetRCString( "TaskbarLeft", bufferLeft, "", MAX_PATH );
		  GetRCString( "TaskbarRight", bufferRight, "", MAX_PATH );

		  skin = new TaskbarSkin( bufferLeft, buffer, bufferRight );
	  }

 	  // taskbar button skin
	  if ( GetRCString( "TaskButtonSkin", buffer, "", MAX_PATH ) )
	  {
		  GetRCString( "TaskButtonLeft", bufferLeft, "", MAX_PATH );
		  GetRCString( "TaskButtonRight", bufferRight, "", MAX_PATH );

		  buttonSkin = new TaskbarSkin( bufferLeft, buffer, bufferRight );
	  }

	  // taskbar button flash skin
	  if ( GetRCString( "TaskButtonFlashSkin", buffer, "", MAX_PATH ) )
	  {
		  GetRCString( "TaskButtonFlashLeft", bufferLeft, "", MAX_PATH );
		  GetRCString( "TaskButtonFlashRight", bufferRight, "", MAX_PATH );

		  flashSkin = new TaskbarSkin( bufferLeft, buffer, bufferRight );
	  }

	  // active taskbar button skin
	  if ( GetRCString( "TaskButtonSkinActive", buffer, "", MAX_PATH ) )
	  {
		  GetRCString( "TaskButtonLeftActive", bufferLeft, "", MAX_PATH );
		  GetRCString( "TaskButtonRightActive", bufferRight, "", MAX_PATH );

		  activeButtonSkin = new TaskbarSkin( bufferLeft, buffer, bufferRight );
	  }

	  // minimized taskbar button skin
	  if ( GetRCString( "TaskButtonSkinMin", buffer, "", MAX_PATH ) )
	  {
		  GetRCString( "TaskButtonLeftMin", bufferLeft, "", MAX_PATH );
		  GetRCString( "TaskButtonRightMin", bufferRight, "", MAX_PATH );

		  minimizedSkin = new TaskbarSkin( bufferLeft, buffer, bufferRight );
	  }

	}

	// fonts
	NONCLIENTMETRICS ncm, ncm2, ncm3, ncm4;
	ncm4.cbSize = ncm3.cbSize = ncm2.cbSize = ncm.cbSize = sizeof(NONCLIENTMETRICS);

	SystemParametersInfo(SPI_GETNONCLIENTMETRICS, sizeof(NONCLIENTMETRICS), &ncm, 0);
	SystemParametersInfo(SPI_GETNONCLIENTMETRICS, sizeof(NONCLIENTMETRICS), &ncm2, 0);
	SystemParametersInfo(SPI_GETNONCLIENTMETRICS, sizeof(NONCLIENTMETRICS), &ncm3, 0);
	SystemParametersInfo(SPI_GETNONCLIENTMETRICS, sizeof(NONCLIENTMETRICS), &ncm4, 0);
	
	font = GetRCFont("TaskbarFont", &ncm.lfCaptionFont);
	activeFont = GetRCFont("TaskbarActiveFont", &ncm2.lfCaptionFont);
	minimizedFont = GetRCFont("TaskbarMinFont", &ncm3.lfCaptionFont);
	flashFont = GetRCFont("TaskbarFlashFont", &ncm4.lfCaptionFont);

	textColor = GetRCColor( "TaskbarText", msTaskbar ? GetSysColor( COLOR_BTNTEXT ) : RGB( 255, 255, 255 ) );/*wtf?????*/
	textActiveColor = GetRCColor( "TaskbarTextActive", msTaskbar ? GetSysColor( COLOR_BTNTEXT ) : RGB( 0, 0, 0 ) );
    textMinimizedColor = GetRCColor( "TaskbarTextMin", msTaskbar ? GetSysColor( COLOR_BTNTEXT ) : RGB( 0, 0, 0 ) );
    textFlashColor = GetRCColor( "TaskbarTextFlash", msTaskbar ? GetSysColor( COLOR_BTNTEXT ) : RGB( 255, 255, 255 ) );

	taskbarPositioning = GetRCBoolDef("TaskbarPositioning", FALSE);
	if (taskbarPositioning)
	{
		taskbarX = GetRCInt("TaskbarX", 0);
		taskbarY = GetRCInt("TaskbarY", 0);
		if(taskbarX < 0)
		  taskbarX = SCREEN_WIDTH + taskbarX;
		if(taskbarY < 0)
		  taskbarY = SCREEN_HEIGHT + taskbarY;
	}

    maxTaskWidth = GetRCInt( TEXT("TaskbarMaxTaskWidth"), SCREEN_WIDTH );
	
	borderLeft = GetRCInt( TEXT("TaskBarBorderLeft"), 4 );
	borderTop = GetRCInt( TEXT("TaskBarBorderTop"), 4 );
	borderRight = GetRCInt( TEXT("TaskBarBorderRight"), 4 );
	borderBottom = GetRCInt( TEXT("TaskBarBorderBottom"), 4 );

	buttonBorderLeft = GetRCInt( TEXT("TaskBarButtonBorderLeft"), 4 );
	buttonBorderTop = GetRCInt( TEXT("TaskBarButtonBorderTop"), 4 );
	buttonBorderRight = GetRCInt( TEXT("TaskBarButtonBorderRight"), 4 );
	buttonBorderBottom = GetRCInt( TEXT("TaskBarButtonBorderBottom"), 4 );

	spacing = GetRCInt( TEXT("TaskBarSpacing"), 3 );

	showIcon = GetRCBoolDef("TaskbarIconShow", TRUE) ? true : false;

	taskbarHeight = GetRCInt( TEXT("TaskBarHeight"), 0 );
	taskbarLines =  GetRCInt( TEXT("TaskBarLines"), 1 );
	if (taskbarLines <= 0)
	  taskbarLines = 1;

	GetRCLine("TaskBarLButtonDown", taskbarLButtonDown, MAX_LINE_LENGTH, "!none");
	GetRCLine("TaskBarRButtonDown", taskbarRButtonDown, MAX_LINE_LENGTH, "!none");
	GetRCLine("TaskBarMButtonDown", taskbarMButtonDown, MAX_LINE_LENGTH, "!none");
	GetRCLine("TaskBarLButtonUp", taskbarLButtonUp, MAX_LINE_LENGTH, "!none");
	GetRCLine("TaskBarRButtonUp", taskbarRButtonUp, MAX_LINE_LENGTH, "!none");
	GetRCLine("TaskBarMButtonUp", taskbarMButtonUp, MAX_LINE_LENGTH, "!none");

	removeHiddenHwnd = GetRCBoolDef("TaskbarRemoveHidden", TRUE) ? true : false;
}

void Taskbar::Layout()
{
	int buttonCount = buttons.size();

    //sort (buttons.begin(), buttons.end(), greater<int>() );
	for (TaskbarButtonIterator it0 = buttons.begin(); it0 != buttons.end(); ++it0)
	{
		if( removeHiddenHwnd && IsOnTaskbarHiddenList((*it0)->hTask)){
			--buttonCount;
		}
		if(!IsWindow((*it0)->hTask)){//are we sure the list is clean?
			if ( !IsOnTaskbarIgnoreList((*it0)->hTask) ){
				//when trillian closes ...
				//PrintLog( "Deleted - hWnd: %08X", (*it0)->hTask);
				TaskbarButton *button = *it0;
				delete button;
				buttons.erase( it0 );
			   --buttonCount;
			}
		}
	}


	if (buttonCount <= 0)
	{
		// if no buttons, just repaint
		if ( !autoHidden )
			InvalidateRect( hWnd, NULL, FALSE );

		return ;
	}

	int availableWidth = width - borderLeft - borderRight;
	availableWidth = max( availableWidth, 0 );  // cannot be less than zero

	if ( buttonCount > 1 )
	{
		availableWidth = availableWidth - (buttonCount - 1) * spacing;
		availableWidth = max( availableWidth, 0 );
	}

	int widthPerButton = min(availableWidth / buttonCount,maxTaskWidth);

	int x = borderLeft;
	int y = borderTop;

	int buttonHeight = (height - borderTop - borderBottom - (taskbarLines - 1) * spacing) / taskbarLines;

	int lines =0;
	int linebuttonCount;

    if(taskbarLines>1){
		if (!(buttonCount%taskbarLines))
			linebuttonCount = ((buttonCount) / taskbarLines);//problem when 3 lines
		else
			linebuttonCount = ((buttonCount) / taskbarLines) + 1;//problem when 3 lines
	}
	else
	  linebuttonCount = ((buttonCount) / taskbarLines);

    int error = availableWidth % linebuttonCount;
    int errorDone = 0;
    widthPerButton = min(availableWidth / linebuttonCount,maxTaskWidth);

	// set placement for each button
	for (TaskbarButtonIterator it = buttons.begin(); it != buttons.end(); ++it)
	{
		TaskbarButton *button = *it;
		if(((removeHiddenHwnd && !IsOnTaskbarHiddenList((*it)->hTask))) || !removeHiddenHwnd){
		if ( error > 0 )
		{
			++lines;
			if((lines % taskbarLines) == 0){
			  error--;
			  errorDone++;
			}
 		    button->Move( x , y, widthPerButton + 1, buttonHeight );
			x = borderLeft + (lines / taskbarLines) * widthPerButton + (lines / taskbarLines) * spacing + errorDone;
            y = borderTop + (lines % taskbarLines) * buttonHeight + (lines % taskbarLines) * spacing;
		}
		else
		{
			button->Move( x , y, widthPerButton, buttonHeight );
			++lines;
			x = borderLeft + (lines / taskbarLines) * widthPerButton + (lines / taskbarLines) * spacing + errorDone;
			y = borderTop + (lines % taskbarLines) * buttonHeight + (lines % taskbarLines) * spacing;
		}
		}
		else
			button->Move( x , y, 0, 0);
	}

	if ( !autoHidden )
		InvalidateRect( hWnd, NULL, FALSE );
}

void Taskbar::SetWindowPosition()
{
	int realHeight;

	if ( autoHidden )
		realHeight = 2;
	else
		realHeight = neededHeight;

	if (taskbarPositioning)
	{
		SetWindowPos( hWnd, NULL, taskbarX, taskbarY,
		              (screenWidth), neededHeight, SWP_NOACTIVATE | SWP_NOZORDER );
	}
	else
	{
		if ( edge == EDGE_TOP )
		{
			SetWindowPos( hWnd, NULL, 0, realHeight - neededHeight,
			              (screenWidth), neededHeight, SWP_NOACTIVATE | SWP_NOZORDER );

		}
		else
		{
			SetWindowPos( hWnd, NULL, 0, screenHeight - realHeight,
			              (screenWidth), neededHeight, SWP_NOACTIVATE | SWP_NOZORDER );

		}
	}
}

TaskbarButton *Taskbar::SearchForButtonByHandle( HWND hWnd )
{
	for (TaskbarButtonIterator it = buttons.begin(); it != buttons.end(); ++it)
	{
		if ( hWnd == (*it)->GetTask() )
			return (*it);
	}

	return NULL;
}

void Taskbar::OnActivate( BOOL becomingActive )
{
	active = becomingActive;

	if ( becomingActive && autoHidden )
	{
		autoHidden = FALSE;
		SetWindowPosition();
	}
	else if ( !becomingActive && autoHide )
	{
		// only hide if mouse is also not over taskbar
		if ( !mouseOver )
		{
			autoHidden = TRUE;
			SetWindowPosition();
		}
	}
}

void Taskbar::OnDisplayChange()
{
	/*if ( SCREEN_HEIGHT < screenHeight){
		screenHeight = SCREEN_HEIGHT;
		screenWidth = SCREEN_WIDTH;
	}

	SetWindowPosition();*/
}

void Taskbar::OnGetMinRect( HWND hWnd, RECT *rectangle )
{
/*		char className[64];
		GetClassName( hWnd, className, 64 );
	 
		char title[256];
		GetWindowText( hWnd, title, 256 );
	 
		PrintLog( "GetMinRect - hWnd: %08X; className: %s; title: %s",
			hWnd, className, title ); */
}

void Taskbar::OnNewTask( HWND hWnd, RECT *rectangle )
{
/*		char className[64];
		GetClassName( hWnd, className, 64 );
	 
		char title[256];
		GetWindowText( hWnd, title, 256 );
	 
		PrintLog( "OnNewTask - hWnd: %08X; className: %s; title: %s",
			hWnd, className, title ); */
}

void Taskbar::OnTaskDestroy( HWND hWnd, RECT *rectangle )
{
/*		char className[64];
		GetClassName( hWnd, className, 64 );
	 
		char title[256];
		GetWindowText( hWnd, title, 256 );
	 
		PrintLog( "OnTaskDestroy - hWnd: %08X; className: %s; title: %s",
			hWnd, className, title ); */
}

void Taskbar::OnLButtonDown( int x, int y )
{
	TaskbarButton *button;

	if ( capture )
		button = capture;
	else
		button = ButtonAtPoint( x, y );

	if ( button )
	{
		button->OnLButtonDown( x, y );
	}
    
	else
	{
		LSExecute(NULL, Taskbar::taskbarLButtonDown, NULL);
	}
	
}

void Taskbar::OnLButtonUp( int x, int y )
{
	TaskbarButton *button;

	if ( capture )
		button = capture;
	else
		button = ButtonAtPoint( x, y );

	if ( button )
		button->OnLButtonUp( x, y );
	else
	{
		LSExecute(NULL, Taskbar::taskbarLButtonUp, NULL);
	}

}

void Taskbar::OnMButtonDown( int x, int y )
{
	TaskbarButton *button;

	if ( capture )
		button = capture;
	else
		button = ButtonAtPoint( x, y );

	if ( button )
	{
			button->OnMButtonDown( x, y );
	}
	else
	{
		LSExecute(NULL, Taskbar::taskbarMButtonDown, NULL);
	}
}

void Taskbar::OnMButtonUp( int x, int y )
{
	TaskbarButton *button;

	if ( capture )
		button = capture;
	else
		button = ButtonAtPoint( x, y );

	if ( button )
		button->OnMButtonUp( x, y );
	else
	{
		LSExecute(NULL, Taskbar::taskbarMButtonUp, NULL);
	}

}



void Taskbar::OnMouseMove( int x, int y )
{
	mouseOver = TRUE;

	// is autohide enabled?
	if ( autoHide )
	{
		// are we hidden?
		if ( autoHidden )
		{
			// if hidden, unhide
			autoHidden = FALSE;
			SetWindowPosition();

			// timer to keep us visible
			SetTimer( hWnd, TIMER_AUTOHIDE, autoHideDelay, NULL );

			return ;
		}

		// timer to keep us visible
		SetTimer( hWnd, TIMER_AUTOHIDE, autoHideDelay, NULL );
	}

	TaskbarButton *button;

	if ( capture )
		button = capture;
	else
		button = ButtonAtPoint( x, y );

	if ( button )
	{
		button->OnMouseMove( x, y );
		return ;
	}

	POINT pt;
	RECT r;

	pt.x = x;
	pt.y = y;

	r.left = 2;
	r.top = 4;
	r.right = r.left + 64;
	r.bottom = height - 4;

}

void Taskbar::OnPaint( HDC hDC )
{
	HDC hdcBuffer;
	HBITMAP hbmBuffer;
	HBITMAP hbmTemp;

	hdcBuffer = CreateCompatibleDC( hDC );
	hbmBuffer = CreateCompatibleBitmap( hDC, width, height );
	
	hbmTemp = (HBITMAP) SelectObject( hdcBuffer, hbmBuffer );

	if ( !skin )
	{
		RECT r;
		GetClientRect( hWnd, &r );

		if ( !msTaskbar )
		{
			// simple 3D frame
			HBRUSH hbrBack;
			HPEN hpnFore;
			HPEN hpnTemp;

			hbrBack = CreateSolidBrush( backColor );
			FillRect( hdcBuffer, &r, hbrBack );
			DeleteObject( hbrBack );

			hpnFore = CreatePen( PS_SOLID, 1, foreColor2 );
			hpnTemp = (HPEN) SelectObject( hdcBuffer, hpnFore );

			MoveToEx( hdcBuffer, r.left, r.bottom - 1, NULL );
			LineTo( hdcBuffer, r.right - 1, r.bottom - 1 );
			LineTo( hdcBuffer, r.right - 1, r.top );

			SelectObject( hdcBuffer, hpnTemp );
			DeleteObject( hpnFore );

			hpnFore = CreatePen( PS_SOLID, 1, foreColor1 );
			hpnTemp = (HPEN) SelectObject( hdcBuffer, hpnFore );

			LineTo( hdcBuffer, r.left, r.top );
			LineTo( hdcBuffer, r.left, r.bottom - 1 );

			SelectObject( hdcBuffer, hpnTemp );
			DeleteObject( hpnFore );
		}
		else
		{
			// MS taskbar style
			DrawEdge( hdcBuffer, &r, EDGE_RAISED, BF_RECT | BF_MIDDLE );
		}
	}
	else
	{
		// skin
		skin->Apply( hdcBuffer, 0, 0, width, height );
	}

	for (TaskbarButtonIterator it = buttons.begin(); it != buttons.end(); ++it)
		if(removeHiddenHwnd && !IsOnTaskbarHiddenList((*it)->hTask))
			(*it)->OnPaint( hdcBuffer );
		else if(!removeHiddenHwnd)
			(*it)->OnPaint( hdcBuffer );

	BitBlt( hDC, 0, 0, width, height, hdcBuffer, 0, 0, SRCCOPY );
	

	SelectObject( hdcBuffer, hbmTemp );
	DeleteObject( hbmBuffer );
	DeleteDC( hdcBuffer );
}

// OnRedraw
//

void Taskbar::OnRedraw( HWND hWnd, BOOL fFlash )
{
	TaskbarButton *button = SearchForButtonByHandle(hWnd);

	if (button)
	{
		button->SetFlashing(fFlash);
		button->OnUpdate(TRUE);
	}
}

void Taskbar::OnRButtonDown( int x, int y )
{
	TaskbarButton *button;

	if ( capture )
		button = capture;
	else
		button = ButtonAtPoint( x, y );

	if ( button )
		button->OnRButtonDown( x, y );
	else
      LSExecute(NULL, this->taskbarRButtonDown, NULL);

}

void Taskbar::OnRButtonUp( int x, int y )
{
	TaskbarButton *button;

	if ( capture )
		button = capture;
	else
		button = ButtonAtPoint( x, y );

	if ( button )
		button->OnRButtonUp( x, y );
	else{
		LSExecute(NULL, Taskbar::taskbarRButtonUp, NULL);
	}
}

void Taskbar::OnSettingChange()
{
	// compute needed height and set initial position
	TEXTMETRIC tm;
	HDC hDC;
	HFONT hfnTemp;

	hDC = GetDC( NULL );
	hfnTemp = (HFONT) SelectObject( hDC, font );

	GetTextMetrics( hDC, &tm );

	SelectObject( hDC, hfnTemp );
	ReleaseDC( NULL, hDC );

	if(taskbarHeight <= 0){
	  neededHeight = max( (const int)(tm.tmHeight + tm.tmExternalLeading), iconSize );
  	  neededHeight = neededHeight + buttonBorderTop + buttonBorderBottom;
	  neededHeight = neededHeight + borderTop + borderBottom;
	}
	else
      neededHeight = taskbarHeight;

	screenHeight = SCREEN_HEIGHT;

	char widthStr[256];
	GetRCString("TaskbarWidth", widthStr, NULL, 256);
	int lastChar = strlen(widthStr) - 1;
	if (widthStr != NULL && strlen(widthStr) > 0 && widthStr[lastChar] == '%')
	{
		// percentage width
		widthStr[lastChar] = 0;
		screenWidth = int(((double)strtol(widthStr, NULL, 0) / 100.0) * (double)SCREEN_WIDTH);
	}
	else
	{
		// absolute width or default to screen width
		int blah = SCREEN_WIDTH;
		screenWidth = GetRCInt("TaskbarWidth", SCREEN_WIDTH);
	}
	savedScreenWidth = screenWidth;//jesus
	SetWindowPosition();
}

void Taskbar::OnSize( int widthP, int heightP )
{
	height = heightP;
	width = widthP;
	Layout();
}


void Taskbar::OnReSize( int widthP, int heightP )
{
	height = heightP;
	width = widthP;
	
	neededHeight = height;
    screenWidth = width;

    SetWindowPosition();
}

void Taskbar::OnMove( int xP, int yP )
{
	taskbarX = xP;
	taskbarY = yP;
    SetWindowPosition();
	Layout();
}


void Taskbar::OnTimerAutoHide()
{
	POINT pt;
	RECT r;

	GetCursorPos( &pt );
	GetWindowRect( hWnd, &r );

	if ( !PtInRect( &r, pt ) )
	{
		// if mouse moved off window, autohide
		KillTimer( hWnd, TIMER_AUTOHIDE );
		mouseOver = FALSE;

		if ( !active )
		{
			// only hide if we're also not active
			autoHidden = TRUE;
			SetWindowPosition();
		}
	}
}

void Taskbar::OnTimerUpdate()
{
	listChanged = false;

	// add new windows
	EnumWindows(Taskbar::EnumWindowsProc, (LPARAM) this);

	// if a window no longer exists, _tremove it... otherwise update it
	for (int i = 0; i < buttons.size(); i++)
	{
		if (IsWindow(buttons[i]->GetTask()))
		{
			buttons[i]->OnUpdate();
		}
		else
		{
			TaskbarButton *button = buttons[i];
			buttons.erase(&button);
			delete button;

			listChanged = true;
		}
	}

	// if task list changed, relayout
	if (listChanged)
		Layout();
}

void Taskbar::OnWindowActivated( HWND hApplication, BOOL fFullScreen )
{
	// update buttons
	for (TaskbarButtonIterator it = buttons.begin(); it != buttons.end(); ++it)
		(*it)->OnUpdate();
}

void Taskbar::OnWindowCreated( HWND hTask )
{
	/*RECT r;
	OnNewTask( hTask, &r);//debug*/
    if ( !IsOnTaskbarIgnoreList(hTask)){
			TaskbarButton *button = new TaskbarButton( *this, hTask );
			button->SetSkin(buttonSkin);
			button->SetActiveSkin(activeButtonSkin);
			button->SetFlashSkin(flashSkin);
			button->SetMinimizedSkin(minimizedSkin);

			buttons.push_back( button );

			Layout();
	}
	else if ((taskbarTrayResize != SIZE_NO) && (IsSystrayHwnd(hWnd))){
	    RECT r;
        GetClientRect( hWnd, &r );

		if(taskbarTrayResize == SIZE_RIGHT)
			taskbar->OnReSize( taskbar->savedScreenWidth - r.right, neededHeight );
		else{
			taskbarX = taskbarX + (width - taskbar->savedScreenWidth + r.right);
 	        taskbarY = taskbarY;
 	        SetWindowPosition();
			//taskbar->OnMove(taskbarX + (width - taskbar->savedScreenWidth + r.right),taskbarY);
			taskbar->OnReSize( taskbar->savedScreenWidth - r.right, r.bottom - r.top );
		}
	}
}

void Taskbar::OnWindowDestroyed( HWND hTask )
{

	/*RECT r;
	OnTaskDestroy( hTask, &r);//debug*/
	for (TaskbarButtonIterator i = buttons.begin(); i != buttons.end(); ++i )
	{
		TaskbarButton *button = *i;

		if ( button->GetTask() == hTask )
		{
			if ( !IsOnTaskbarIgnoreList(hTask) ){
					delete button;
					buttons.erase( i );

					Layout();
			}
	        return ;
		}
	}
	return ;
}

BOOL Taskbar::OnWindowMessage( UINT nMessage, WPARAM wParam, LPARAM lParam, LRESULT &lResult )
{
	lResult = 0;

	// relay mouse messages to tooltips control
	if ( nMessage >= WM_MOUSEFIRST && nMessage <= WM_MOUSELAST )
	{
		MSG msg;

		msg.hwnd = hWnd;
		msg.message = nMessage;
		msg.wParam = wParam;
		msg.lParam = lParam;

		SendMessage( hToolTips, TTM_RELAYEVENT, 0, (LPARAM) &msg );
	}
	switch ( nMessage )
	{
		case LM_LSSELECT:
		{
			OnWindowActivated( NULL, FALSE );
			return TRUE;
		}

		case LM_GETMINRECT:
		{
			OnGetMinRect( (HWND) wParam, (RECT *) lParam );
			return TRUE;
		}

		case LM_REDRAW:
		{
			OnRedraw( (HWND) wParam, (BOOL) lParam );
			return TRUE;
		}

		case LM_WINDOWACTIVATED:
		{
			OnWindowActivated( (HWND) wParam, (BOOL) lParam );
			return TRUE;
		}

		case LM_WINDOWCREATED:
		{
			OnWindowCreated( (HWND) wParam );
			return TRUE;
		}

		case LM_WINDOWDESTROYED:
		{
			OnWindowDestroyed( (HWND) wParam );
			return TRUE;
		}

		case LM_GETREVID:
		{
			char* buf = (char*)lParam;
			strcpy(buf, aboutStr);
			return strlen(buf);
		}

		case WM_ACTIVATE:
		{
			OnActivate( (LOWORD(wParam) == WA_INACTIVE) ? FALSE : TRUE );
			return TRUE;
		}

		case WM_DISPLAYCHANGE:
		{
			OnDisplayChange();
			return TRUE;
		}

		case WM_LBUTTONDOWN:
		{
			OnLButtonDown( LOWORD(lParam), HIWORD(lParam));
			return TRUE;
		}

		case WM_LBUTTONUP:
		{
			OnLButtonUp( LOWORD(lParam), HIWORD(lParam));
			return TRUE;
		}

		case WM_MOUSEMOVE:
		{
			OnMouseMove( LOWORD( lParam ), HIWORD( lParam ) );
			return TRUE;
		}

		case WM_PAINT:
		{
			PAINTSTRUCT ps;
			HDC hDC;

			if ( !wParam )
				hDC = BeginPaint( hWnd, &ps );
			else
				hDC = (HDC) wParam;

			OnPaint( hDC );

			if ( !wParam )
				EndPaint( hWnd, &ps );

			return TRUE;
		}

		case WM_RBUTTONDOWN:
		{
			OnRButtonDown( (int) (short) LOWORD( lParam ), (int) (short) HIWORD( lParam ) );
			return TRUE;
		}

		case WM_RBUTTONUP:
		{
			OnRButtonUp( (int) (short) LOWORD( lParam ), (int) (short) HIWORD( lParam ) );
			return TRUE;
		}

		case WM_SETTINGCHANGE:
		{
			OnSettingChange();
			return TRUE;
		}

		case WM_SIZE:
		{
			OnSize( (int) (short) LOWORD( lParam ), (int) (short) HIWORD( lParam ) );
			return TRUE;
		}

		case WM_TIMER:
		{
			switch ( wParam )
			{
				case TIMER_AUTOHIDE:
				{
					OnTimerAutoHide();
					return TRUE;
				}

				
				case TIMER_UPDATE:
				{
					OnTimerUpdate();
					return TRUE;
				}
				
			}

		return FALSE;
		}

		case WM_MBUTTONDOWN:
		{
			OnMButtonDown( (int) (short) LOWORD( lParam ), (int) (short) HIWORD( lParam ) );
			return TRUE;
		}

		case WM_MBUTTONUP:
		{
			OnMButtonUp( (int) (short) LOWORD( lParam ), (int) (short) HIWORD( lParam ) );
			return TRUE;
		}
	}
	return FALSE;
}

BOOL WINAPI Taskbar::EnumWindowsProc(HWND hWnd, LPARAM lParam)
{
	Taskbar *taskbar = (Taskbar *) lParam;

	if (taskbar && IsAppWindow(hWnd))
	{
        if( !IsOnTaskbarIgnoreList(hWnd) ){
			TaskbarButton *button = taskbar->SearchForButtonByHandle(hWnd);
			if (button == 0)
			{
				button = new TaskbarButton(*taskbar, hWnd);

				button->SetActiveSkin(taskbar->activeButtonSkin);
				button->SetSkin(taskbar->buttonSkin);
				button->SetFlashSkin(taskbar->flashSkin);
				button->SetMinimizedSkin(taskbar->minimizedSkin);

				taskbar->buttons.push_back(button);
				taskbar->listChanged = true;
			}
		}
	}
	else if ((taskbar->taskbarTrayResize != SIZE_NO) && (IsSystrayHwnd(hWnd))){
	    RECT r;
        GetClientRect( hWnd, &r );

		if(taskbar->taskbarTrayResize == SIZE_RIGHT)
			taskbar->OnReSize( taskbar->savedScreenWidth - r.right, taskbar->neededHeight );
		else{
			taskbar->taskbarX = taskbar->taskbarX + (taskbar->width - taskbar->savedScreenWidth + r.right);
 	        taskbar->taskbarY = taskbar->taskbarY;
 	        taskbar->SetWindowPosition();

			//taskbar->OnMove(taskbar->taskbarX + (taskbar->width - taskbar->savedScreenWidth + r.right),taskbar->taskbarY);
			taskbar->OnReSize( taskbar->savedScreenWidth - r.right, r.bottom - r.top );
		}
	}

	return TRUE;
}

BOOL Taskbar::SystrayResize(HWND hWnd){//j666
	if (IsSystrayHwnd(hWnd)){
	    RECT r;
        GetClientRect( hWnd, &r );

	    taskbar->OnReSize( taskbar->savedScreenWidth - r.right, r.bottom - r.top );
	}

	return TRUE;
}

LRESULT WINAPI Taskbar::WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	Taskbar *taskbar = (Taskbar *) GetWindowLong(hWnd, 0);

	if (message == WM_NCCREATE)
	{
		LPCREATESTRUCT pcs = (LPCREATESTRUCT) lParam;
		taskbar = (Taskbar *) pcs->lpCreateParams;

		taskbar->hWnd = hWnd;
		SetWindowLong(hWnd, 0, (LONG) taskbar);
	}

	if (taskbar)
	{
		LRESULT lResult;

		if (taskbar->OnWindowMessage(message, wParam, lParam, lResult))
			return lResult;
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}

//
// TaskbarButton
//

TaskbarButton::TaskbarButton( Taskbar &tb, HWND hWnd )
		: container( tb )
{
	hTask = hWnd;
	skin = NULL;
	activeskin = NULL;
	flashskin = NULL;
	minimizedskin = NULL;
	otherCaption = NULL;
	caption = NULL;

	active = (GetForegroundWindow() == hTask);
	if (hTask)
	{
		caption = GetWindowCaption();
		// add tooltip
		TOOLINFO ti;
		memset( &ti, 0, sizeof(TOOLINFO) );

		ti.cbSize = sizeof(TOOLINFO);
		ti.hwnd = container.hWnd;
		ti.uId = (UINT) this;
		ti.lpszText = caption;

		SendMessage( container.hToolTips, TTM_ADDTOOL, 0, (LPARAM) &ti );
	}

	if(taskbar->showIcon)
	    icon = GetIcon();

	mousePressed = FALSE;
	mouseOver = FALSE;
	taskflash = FALSE;

	x = 0;
	y = 0;
	height = 0;
	width = 0;
}

TaskbarButton::~TaskbarButton()
{
	delete caption;
	delete otherCaption;

	// delete tooltip
	TOOLINFO ti;
	memset( &ti, 0, sizeof(TOOLINFO) );

	ti.cbSize = sizeof(TOOLINFO);
	ti.hwnd = container.hWnd;
	ti.uId = (UINT) this;

	SendMessage( container.hToolTips, TTM_DELTOOL, 0, (LPARAM) &ti );
}

BOOL TaskbarButton::HasPoint( int xP, int yP )
{
	if ( xP < (x + width) && xP >= x )
	{
		if ( yP < (y + height) && yP >= y )
			return TRUE;
	}

	return FALSE;
}

void TaskbarButton::Move( int xP, int yP, int widthP, int heightP )
{
	x = xP;
	y = yP;
	height = heightP;
	width = widthP;

	// update tooltip
	TOOLINFO ti;
	memset( &ti, 0, sizeof(TOOLINFO) );

	ti.cbSize = sizeof(TOOLINFO);
	ti.hwnd = container.hWnd;
	ti.uId = (UINT) this;
	ti.rect.left = x;
	ti.rect.top = y;
	ti.rect.right = x + width;
	ti.rect.bottom = y + height;

	SendMessage( container.hToolTips, TTM_NEWTOOLRECT, 0, (LPARAM) &ti );
}

void TaskbarButton::OnLButtonDown( int, int )
{
	container.SetCapture( this );

	mousePressed = TRUE;
	container.Invalidate( x, y, width + 1, height + 1 );
}

void TaskbarButton::OnLButtonUp( int, int )
{
	mousePressed = FALSE;

	if ( container.GetCapture() != this )
		return ;

	container.ReleaseCapture();

	if ( !mouseOver )
	{
		container.Invalidate( x, y, width + 1, height + 1 );
		return ;
	}

	if ( active )
	{
		// already active, minimize it
		ShowWindowAsync( hTask, SW_MINIMIZE );
		active = FALSE;
	}
	else
	{
		if ( !IsIconic( hTask ) )
		{
			// if not minimized, move to application's current desktop
			SendMessage( GetLitestepWnd(), LM_BRINGTOFRONT, 0, (LPARAM) hTask );
		}

		// undocumented call to switch window activation
		SwitchToThisWindow( hTask, 1 );
		active = TRUE;
	}

	OnUpdate(TRUE);
}


void TaskbarButton::OnMButtonDown( int, int )
{
//	container.SetCapture( this );

//	mousePressed = TRUE;
	container.Invalidate( x, y, width + 1, height + 1 );
}

void TaskbarButton::OnMButtonUp( int, int )
{
//	container.SetCapture( this );

//	mousePressed = TRUE;
	container.Invalidate( x, y, width + 1, height + 1 );
}


void TaskbarButton::OnMouseMove( int mX, int mY )
{
	if ( mouseOver != HasPoint( mX, mY ) )
	{
		mouseOver = !mouseOver;
		container.Invalidate( x, y, width + 1, height + 1 );
	}
}

void TaskbarButton::OnPaint( HDC hDC )
{
	RECT r;

	r.left = x;
	r.top = y;
	r.right = x + width;
	r.bottom = y + height;

	TaskbarSkin *tempskin;


	int skinShift = 0;
	int textShift = 0;

	BOOL drawPressed = active || (mouseOver && mousePressed);

	if(GetFlashing() && container.IsTaskFlash())
		tempskin = GetFlashSkin();
	else if(IsIconic(hTask))
		tempskin = GetMinimizedSkin();
	else if ( drawPressed )
	{
		//flash active
		/*if ( GetActiveSkin() )*/
		tempskin = GetActiveSkin();

		skinShift = container.noSkinShift ? 0 : 1;
		textShift = container.noTextShift ? 0 : 1;
	}
	else{
		tempskin = GetSkin();
	}

	if ( !tempskin )
	{
		if ( !container.msTaskbar )
		{
			// simple 3D frame
			HBRUSH hbrBack;
			HPEN hpnFore;
			HPEN hpnTemp;
			COLORREF backcolor;
			
			if(GetFlashing() && container.IsTaskFlash())
				backcolor = container.flashColor;
			if(IsIconic(hTask))
				backcolor = container.minColor;
			else if ( drawPressed )
			{
				backcolor = container.activeColor;
			}
			else{
				backcolor = container.backColor;
			}

			hbrBack = CreateSolidBrush(backcolor);
			FillRect( hDC, &r, hbrBack );
			DeleteObject( hbrBack );

			hpnFore = CreatePen( PS_SOLID, 1, drawPressed ? container.foreColor1 : container.foreColor2 );
			hpnTemp = (HPEN) SelectObject( hDC, hpnFore );

			MoveToEx( hDC, r.left, r.bottom - 1, NULL );
			LineTo( hDC, r.right - 1, r.bottom - 1 );
			LineTo( hDC, r.right - 1, r.top );

			SelectObject( hDC, hpnTemp );
			DeleteObject( hpnFore );

			hpnFore = CreatePen( PS_SOLID, 1, drawPressed ? container.foreColor2 : container.foreColor1 );
			hpnTemp = (HPEN) SelectObject( hDC, hpnFore );

			LineTo( hDC, r.left, r.top );
			LineTo( hDC, r.left, r.bottom - 1 );

			SelectObject( hDC, hpnTemp );
			DeleteObject( hpnFore );
		}
		else
		{
			// MS taskbar style
			DrawEdge( hDC, &r, drawPressed ? EDGE_SUNKEN : EDGE_RAISED, BF_RECT | BF_MIDDLE | BF_SOFT );
		}
	}
	else
	{
		// simple 3D frame
		HBRUSH hbrBack;

		hbrBack = CreateSolidBrush( container.backColor );
		FillRect( hDC, &r, hbrBack );
		DeleteObject( hbrBack );

		// skin
		tempskin->Apply( hDC, x + skinShift, y + skinShift, width, height );
	}

	r.left += taskbar->buttonBorderLeft;
	r.top += taskbar->buttonBorderTop;
	r.right -= taskbar->buttonBorderRight;
	r.bottom -= taskbar->buttonBorderBottom;

	OffsetRect( &r, textShift, textShift );

	if ( taskbar->showIcon && icon && (r.right - r.left) >= taskbar->iconSize )
	{
		int iconY = r.top + ((r.bottom - r.top) - taskbar->iconSize + 1) / 2;

		DrawIconEx( hDC, r.left, iconY, icon, taskbar->iconSize,
		            taskbar->iconSize, 0, NULL, DI_NORMAL );

		r.left = r.left + taskbar->iconSize + taskbar->iconSpacing;
	}

	COLORREF tempColor;
	HFONT tempFont;

	if(GetFlashing() && container.IsTaskFlash())
	{
		tempColor = SetTextColor( hDC, container.textFlashColor );
		tempFont = (HFONT) SelectObject( hDC, container.flashFont );
	}
	if(IsIconic(hTask))
	{
		tempColor = SetTextColor( hDC, container.textMinimizedColor );
		tempFont = (HFONT) SelectObject( hDC, container.minimizedFont );
	}
	else if ( drawPressed )
	{
        tempColor = SetTextColor( hDC, container.textActiveColor );
		tempFont = (HFONT) SelectObject( hDC, container.activeFont );
	}
	else{
	    tempColor = SetTextColor( hDC, container.textColor );
        tempFont = (HFONT) SelectObject( hDC, container.font );
	}

	SetBkMode( hDC, TRANSPARENT );

	if (hTask)
	{
		if (caption)
		{
			DrawText( hDC, caption, strlen( caption ), &r,
			          DT_SINGLELINE | DT_VCENTER | DT_END_ELLIPSIS );
		}
	}
	else
	{
		if (otherCaption)
		{
			DrawText( hDC, otherCaption, strlen( otherCaption ), &r,
			          DT_SINGLELINE | DT_VCENTER | DT_END_ELLIPSIS );
		}
	}


	SetTextColor( hDC, tempColor );
	SelectObject( hDC, tempFont );
}

void TaskbarButton::OnRButtonDown( int, int )
{
	container.SetCapture( this );

	mousePressed = TRUE;
	container.Invalidate( x, y, width + 1, height + 1 );
}

void TaskbarButton::OnRButtonUp( int, int )
{
	mousePressed = FALSE;
	// undocumented call to switch window activation
	SwitchToThisWindow( hTask, 1 );
	if(active)
	active = FALSE;
	else
	active = TRUE;

	if ( container.GetCapture() != this )
		return ;

	container.ReleaseCapture();

	if ( !mouseOver )
	{
		container.Invalidate( x, y, width + 1, height + 1 );
		return ;
	}

	HMENU systemMenu;
	systemMenu = GetSystemMenu( hTask, FALSE );

	WINDOWPLACEMENT wp;
	wp.length = sizeof(WINDOWPLACEMENT);
	GetWindowPlacement( hTask, &wp );

	// restore is enabled only if minimized or maximized (not normal)
	EnableMenuItem( systemMenu, SC_RESTORE, MF_BYCOMMAND |
	                (wp.showCmd != SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

	// move is enabled only if normal
	EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND |
	                (wp.showCmd == SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

	// size is enabled only if normal
	EnableMenuItem( systemMenu, SC_SIZE, MF_BYCOMMAND |
	                (wp.showCmd == SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

	// minimize is enabled only if not minimized
	EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND |
	                (wp.showCmd != SW_SHOWMINIMIZED ? MF_ENABLED : MF_GRAYED) );

	// maximize is enabled only if not maximized
	EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND |
	                (wp.showCmd != SW_SHOWMAXIMIZED ? MF_ENABLED : MF_GRAYED) );

	// let application modify menu
	SendMessage( hTask, WM_INITMENUPOPUP, (WPARAM) systemMenu, MAKELPARAM( 0, TRUE ) );
	SendMessage( hTask, WM_INITMENU, (WPARAM) systemMenu, 0 );

	// display the menu
	POINT pt;
	GetCursorPos( &pt );

	int command = TrackPopupMenu( systemMenu,
	                              TPM_RETURNCMD | TPM_RIGHTBUTTON,
	                              pt.x, pt.y, 0,
	                              container.hWnd,
	                              NULL );

	if ( command )
	{
		if ( command != SC_MINIMIZE && command != SC_CLOSE )
			SetForegroundWindow( hTask );

		PostMessage( hTask, WM_SYSCOMMAND, (WPARAM) command, MAKELPARAM( pt.x, pt.y ) );
	}
	else
	{
		container.Invalidate( x, y, width + 1, height + 1 );
	}
}

void TaskbarButton::OnUpdate(BOOL needRedraw)
{
	//it's not a force update, so let's check if we need to redraw
	if (!needRedraw)
	{
		// has activiation state changed?
		bool tempActive = (GetForegroundWindow() == hTask);

		if ( tempActive != active )
		{
			active = tempActive;
			needRedraw = TRUE;
		}

		// has caption changed?
		LPSTR temp1, temp2;

		if (hTask)
		{
			temp1 = caption;
			temp2 = GetWindowCaption();
		}
		else
		{
			temp1 = otherCaption;
			temp2 = NULL;
		}

		if ( temp2 == NULL || strcmp( temp1, temp2 ) != 0 )
		{
			if (temp2)
			{
				delete [] caption;
				caption = temp2;
				needRedraw = TRUE;
				temp1 = caption;
			}

			// update tooltip
			TOOLINFO ti;
			memset( &ti, 0, sizeof(TOOLINFO) );

			ti.cbSize = sizeof(TOOLINFO);
			ti.hwnd = container.hWnd;
			ti.uId = (UINT) this;
			ti.lpszText = temp1;

			SendMessage( container.hToolTips, TTM_UPDATETIPTEXT, 0, (LPARAM) &ti );
		}
		else
		{
			if (temp2)
			{
				delete [] temp2;
			}
		}


		// has icon changed?
		HICON tempIcon = GetIcon();

		if ( icon != tempIcon )
		{
			icon = tempIcon;
			needRedraw = TRUE;
		}
	}

	if (IsIconic(this->hTask))
		needRedraw = TRUE;

	BOOL drawPressed = active || (mouseOver && mousePressed);
	if (drawPressed)
		needRedraw = TRUE;

	if ( needRedraw )
		container.Invalidate( x, y, width + 1, height + 1 );
}

LPSTR TaskbarButton::GetWindowCaption()
{
	LPSTR windowCaption = NULL;
	int length;

	if (hTask)
	{
		length = GetWindowTextLength( hTask );

		windowCaption = new char[length + 1];
		GetWindowText( hTask, windowCaption, length + 1 );
	}

	return windowCaption;
}

HICON TaskbarButton::GetIcon()
{
	return GetIconFromWindow( hTask, FALSE );
}

//
// TaskbarSkin
//

TaskbarSkin::TaskbarSkin()
{
	TaskbarSkin(NULL, NULL, NULL);
}

TaskbarSkin::TaskbarSkin( LPCSTR pszLeft, LPCSTR pszMiddle, LPCSTR pszRight )
{
	stretch = TRUE;
	backColor = 0;
	hbmLeft = NULL;
	hbmMiddle = NULL;
	hbmRight = NULL;

	if ( pszLeft && pszLeft[0] )
	{
		hbmLeft = LoadLSImage( pszLeft, NULL );
		GetLSBitmapSize( hbmLeft, &leftW, &leftH );
	}

	if ( pszMiddle && pszMiddle[0] )
	{
		hbmMiddle = LoadLSImage( pszMiddle, NULL );
		GetLSBitmapSize( hbmMiddle, &middleW, &middleH );
	}

	if ( pszRight && pszRight[0] )
	{
		hbmRight = LoadLSImage( pszRight, NULL );
		GetLSBitmapSize( hbmRight, &rightW, &rightH );
	}
}

TaskbarSkin::~TaskbarSkin()
{
	if ( hbmLeft )
		DeleteObject( hbmLeft );

	if ( hbmMiddle )
		DeleteObject( hbmMiddle );

	if ( hbmRight )
		DeleteObject( hbmRight );
}

void TaskbarSkin::Apply( HDC hDC, int x, int y, int width, int height )
{
	HDC hdcBuffer;
	HDC hdcMem;
	HBITMAP hbmBuffer;
	HGDIOBJ hgoBuffer;
	HGDIOBJ hgoMem;
	int widthLeft = width;

	hdcBuffer = CreateCompatibleDC( hDC );
	hbmBuffer = CreateCompatibleBitmap( hDC, width, height );
	hgoBuffer = SelectObject( hdcBuffer, hbmBuffer );

	hdcMem = CreateCompatibleDC( hDC );
	hgoMem;

	SetStretchBltMode( hdcBuffer, STRETCH_DELETESCANS );

	if (backColor)
	{
		// simple 3D frame
		RECT r;
		HBRUSH hbrBack;

		r.top = 0;
		r.left = 0;
		r.bottom = height;
		r.right = width;

		hbrBack = CreateSolidBrush( backColor );
		FillRect( hdcMem, &r, hbrBack );
		FillRect( hdcBuffer, &r, hbrBack );
		DeleteObject( hbrBack );
	}

	if ( hbmLeft && width > 0 )
	{
		hgoMem = SelectObject( hdcMem, hbmLeft );

		if (stretch)
		{
			StretchBlt( hdcBuffer, 0, 0, min( widthLeft, leftW ), height,
			            hdcMem, 0, 0, leftW, leftH, SRCCOPY );
		}
		else
		{
			BitBlt(hdcBuffer, 0, 0, min(widthLeft, leftW), height, hdcMem, 0, 0, SRCCOPY);
		}

		SelectObject( hdcMem, hgoMem );
		widthLeft = widthLeft - leftW;
	}

	if ( hbmRight && width > 0 )
	{
		hgoMem = SelectObject( hdcMem, hbmRight );

		if (stretch)
		{
			StretchBlt( hdcBuffer, width - rightW, 0, min( widthLeft, rightW ),
			            height, hdcMem, 0, 0, rightW, rightH, SRCCOPY );
		}
		else
		{
			BitBlt(hdcBuffer, width - rightW, 0, min(widthLeft, rightW), height, hdcMem, 0, 0, SRCCOPY);
		}

		SelectObject( hdcMem, hgoMem );
		widthLeft = widthLeft - rightW;
	}

	if ( hbmMiddle && width > 0 )
	{
		hgoMem = SelectObject( hdcMem, hbmMiddle );

		if (stretch)
		{
			StretchBlt( hdcBuffer, hbmLeft ? leftW : 0, 0, widthLeft, height,
			            hdcMem, 0, 0, middleW, middleH, SRCCOPY );
		}
		else
		{
			BitBlt(hdcBuffer, hbmLeft ? leftW : 0, 0, widthLeft, height, hdcMem, 0, 0, SRCCOPY);
		}

		SelectObject( hdcMem, hgoMem );
	}

	TransparentBltLS( hDC, x, y, width, height, hdcBuffer, 0, 0,
	                  RGB( 255, 0, 255 ) );

	SelectObject( hdcBuffer, hgoBuffer );
	DeleteObject( hbmBuffer );
	DeleteDC( hdcBuffer );

	DeleteDC( hdcMem );
}

void TaskbarButton::SetSkin(TaskbarSkin *s)
{
	skin = s;
}

void TaskbarButton::SetActiveSkin(TaskbarSkin *s)
{
	activeskin = s;
}

void TaskbarButton::SetFlashSkin(TaskbarSkin *s)
{
	flashskin = s;
}

void TaskbarButton::SetMinimizedSkin(TaskbarSkin *s)
{
	minimizedskin = s;
}

TaskbarSkin* TaskbarButton::GetSkin()
{
	return skin;
}

TaskbarSkin* TaskbarButton::GetActiveSkin()
{
	return activeskin;
}

TaskbarSkin* TaskbarButton::GetFlashSkin()
{
	return flashskin;
}

TaskbarSkin* TaskbarButton::GetMinimizedSkin()
{
	return minimizedskin;
}

void TaskbarSkin::SetStretch(BOOL s)
{
	stretch = s;
}

BOOL TaskbarSkin::GetStretch()
{
	return stretch;
}

void TaskbarSkin::SetBackColor(COLORREF color)
{
	backColor = color;
}

COLORREF TaskbarSkin::GetBackColor()
{
	return backColor;
}

void TaskbarButton::SetCaption(LPCSTR cap)
{
	if (!cap)
	{
		return ;
	}

	if (otherCaption)
	{
		delete [] otherCaption;
	}

	otherCaption = new char[strlen(cap) + 1];

	strcpy(otherCaption, cap);

	// add tooltip
	TOOLINFO ti;
	memset( &ti, 0, sizeof(TOOLINFO) );

	ti.cbSize = sizeof(TOOLINFO);
	ti.hwnd = container.hWnd;
	ti.uId = (UINT) this;
	ti.lpszText = otherCaption;

	SendMessage( container.hToolTips, TTM_ADDTOOL, 0, (LPARAM) &ti );
}
